from liblaf.cherries import integration


def default(exp: integration.Experiment) -> integration.Experiment:
    exp.add_tag.add(
        integration.comet.AddTag(),
    )
    exp.add_tags.add(
        integration.comet.AddTags(),
    )
    exp.end.add(
        integration.git.End(),
        integration.dvc.End(),
        integration.logging.End(),
        integration.comet.End(),
    )
    exp.log_asset.add(
        integration.dvc.LogAsset(),
        integration.comet.LogAsset(),
    )
    exp.log_code.add(
        integration.comet.LogCode(),
    )
    exp.log_metric.add(
        integration.comet.LogMetric(),
    )
    exp.log_metrics.add(
        integration.comet.LogMetrics(),
    )
    exp.log_other.add(
        integration.comet.LogOther(),
    )
    exp.log_others.add(
        integration.comet.LogOthers(),
    )
    exp.log_param.add(
        integration.comet.LogParam(),
    )
    exp.log_params.add(
        integration.comet.LogParams(),
    )
    exp.start.add(
        integration.logging.Start(),
        integration.comet.Start(),
    )
    return integration.exp



from typing import override

import attrs
import loguru
from loguru import logger

from liblaf import grapes

from . import _abc
from ._exp import exp


@attrs.define
class End(_abc.End):
    priority: int = attrs.field(default=-1, kw_only=True)  # before Comet
    text: bool = True
    jsonl: bool = False

    @override
    def __call__(self, **kwargs) -> None:
        logger.complete()
        if self.text and (exp.exp_dir / "run.log").exists():
            exp.log_asset(exp.exp_dir / "run.log")
        if self.jsonl and (exp.exp_dir / "run.log.jsonl").exists():
            exp.log_asset(exp.exp_dir / "run.log.jsonl")


@attrs.define
class Start(_abc.Start):
    priority: int = attrs.field(default=-1, kw_only=True)  # before Comet
    text: bool = True
    jsonl: bool = False

    @override
    def __call__(self, **kwargs) -> None:
        handlers: list[loguru.HandlerConfig] = [
            grapes.logging.rich_handler(enable_link=False)
        ]
        if self.text:
            handlers.append(grapes.logging.file_handler(sink=exp.exp_dir / "run.log"))
        if self.jsonl:
            handlers.append(
                grapes.logging.jsonl_handler(sink=exp.exp_dir / "run.log.jsonl")
            )
        grapes.init_logging(handlers=handlers)



import subprocess as sp
from pathlib import Path
from typing import override

import attrs
import git
import git.exc

from liblaf.cherries import pathutils as _path
from liblaf.cherries.typed import PathLike

from . import _abc


@attrs.define
class End(_abc.End):
    priority: int = attrs.field(default=-2, kw_only=True)  # after logging

    @override
    def __call__(self, **kwargs) -> None:
        sp.run(["dvc", "status"], check=True)
        sp.run(["dvc", "push"], check=True)


@attrs.define
class LogAsset(_abc.LogAsset):
    priority: int = attrs.field(default=-1, kw_only=True)  # before Comet

    @override
    def __call__(self, path: PathLike, prefix: str | None = None, **kwargs) -> None:
        path: Path = _path.as_path(path)
        if check_ignore(path) or tracked_by_git(path):
            return
        sp.run(["dvc", "add", "--quiet", path], check=True)


def check_ignore(path: PathLike) -> bool:
    proc: sp.CompletedProcess[bytes] = sp.run(
        ["dvc", "check-ignore", "--quiet", path], check=False
    )
    return proc.returncode == 0


def tracked_by_git(path: PathLike) -> bool:
    path: Path = _path.as_path(path).absolute()
    try:
        repo = git.Repo(search_parent_directories=True)
        repo.git.ls_files(path, error_unmatch=True)
    except git.exc.GitCommandError:
        return False
    return True
